@@ -16,7 +16,7 @@ import struct,  sys,  math
 # corresponds to one beat. This number is somewhat arbitrary, but should be chosen
 # to provide adequate temporal resolution.
 
-TICKSPERBEAT = 128
+TICKSPERBEAT = 960
 
 controllerEventTypes = {
                         'pan' : 0x0a
@@ -89,7 +89,6 @@ class GenericEvent():
                 return False
         if self.type == 'controllerEvent':
             if self.parameter1 != other.parameter1 or \
-                self.parameter2 != other.parameter2 or \
                 self.channel != other.channel or \
                 self.eventType != other.eventType:
                 return False
@@ -453,6 +452,36 @@ class MIDITrack:
         '''
         Write the events in MIDIEvents to the MIDI stream.
         '''
+        preciseTime = 0.0                   # Actual time of event, ignoring round-off
+        actualTime = 0.0                    # Time as written to midi stream, include round-off
+        for event in self.MIDIEventList:
+
+            preciseTime = preciseTime + event.time
+
+            # Convert the time to variable length and back, to see how much
+            # error is introduced
+
+            testBuffer = ""
+            varTime = writeVarLength(event.time)
+            for timeByte in varTime:
+                testBuffer = testBuffer + struct.pack('>B',timeByte)
+            (roundedVal,discard) = readVarLength(0,testBuffer)
+            roundedTime = actualTime + roundedVal
+            # print "Rounded, Precise: %15.10f %15.10f" % (roundedTime, preciseTime)
+
+            # Calculate the delta between the two and apply it to the event time.
+
+            delta = preciseTime - roundedTime
+            event.time = event.time + delta
+
+            # Now update the actualTime value, using the updated event time.
+
+            testBuffer = ""
+            varTime = writeVarLength(event.time)
+            for timeByte in varTime:
+                testBuffer = testBuffer + struct.pack('>B',timeByte)
+            (roundedVal,discard) = readVarLength(0,testBuffer)
+            actualTime = actualTime + roundedVal
         
         for event in self.MIDIEventList:
             if event.type == "NoteOn":
@@ -940,7 +969,7 @@ def writeVarLength(i):
     significant bit is 1, then more bytes follow. If it is zero, then the
     byte in question is the last in the stream
     '''
-    input = int(i)
+    input = int(i+0.5)
     output = [0,0,0,0]
     reversed = [0,0,0,0]
     count = 0
@@ -961,6 +990,27 @@ def writeVarLength(i):
     reversed[3] = output[0]
     return reversed[4-count:4]
 
+# readVarLength is taken from the MidiFile class.
+
+def readVarLength(offset, buffer):
+    '''A function to read a MIDI variable length variable.
+
+    It returns a tuple of the value read and the number of bytes processed. The
+    input is an offset into the buffer, and the buffer itself.
+    '''
+    toffset = offset
+    output = 0
+    bytesRead = 0
+    while True:
+        output = output << 7
+        byte = struct.unpack_from('>B',buffer,toffset)[0]
+        toffset = toffset + 1
+        bytesRead = bytesRead + 1
+        output = output + (byte & 127)
+        if (byte & 128) == 0:
+            break
+    return (output, bytesRead)
+
 def frequencyTransform(freq):
     '''Returns a three-byte transform of a frequencyTransform
     '''
